@@ -13,7 +13,6 @@ type.
 from __future__ import absolute_import
 
 import collections
-import functools
 import inspect
 import io
 import os
@@ -22,7 +21,7 @@ import re
 import sys
 import warnings
 
-from six import string_types, text_type, PY2, PY3
+from six import string_types, text_type, PY2
 
 import _sass
 from sassutils._compat import collections_abc
@@ -32,7 +31,7 @@ __all__ = (
     'SassError', 'SassFunction', 'SassList', 'SassMap', 'SassNumber',
     'SassWarning', 'and_join', 'compile', 'libsass_version',
 )
-__version__ = '0.17.0'
+__version__ = '0.20.1'
 libsass_version = _sass.libsass_version
 
 
@@ -53,7 +52,7 @@ MODES = frozenset(('string', 'filename', 'dirname'))
 
 
 def to_native_s(s):
-    if isinstance(s, bytes) and PY3:  # pragma: no cover (py3)
+    if isinstance(s, bytes) and not PY2:  # pragma: no cover (py3)
         s = s.decode('UTF-8')
     elif isinstance(s, text_type) and PY2:  # pragma: no cover (py2)
         s = s.encode('UTF-8')
@@ -146,8 +145,10 @@ class SassFunction(object):
         if not isinstance(name, string_types):
             raise TypeError('name must be a string, not ' + repr(name))
         elif not isinstance(arguments, collections_abc.Sequence):
-            raise TypeError('arguments must be a sequence, not ' +
-                            repr(arguments))
+            raise TypeError(
+                'arguments must be a sequence, not ' +
+                repr(arguments),
+            )
         elif not callable(callable_):
             raise TypeError(repr(callable_) + ' is not callable')
         self.name = name
@@ -195,9 +196,21 @@ def _normalize_importer_return_value(result):
 
 
 def _importer_callback_wrapper(func):
-    @functools.wraps(func)
-    def inner(path):
-        ret = func(path.decode('UTF-8'))
+    def inner(path, prev):
+        path, prev = path.decode('UTF-8'), prev.decode('UTF-8')
+        num_args = getattr(inner, '_num_args', None)
+        if num_args is None:
+            try:
+                ret = func(path, prev)
+            except TypeError:
+                inner._num_args = 1
+                ret = func(path)
+            else:
+                inner._num_args = 2
+        elif num_args == 2:
+            ret = func(path, prev)
+        else:
+            ret = func(path)
         return _normalize_importer_return_value(ret)
     return inner
 
@@ -477,8 +490,10 @@ def compile(**kwargs):
     A priority of zero is acceptable; priority determines the order callbacks
     are attempted.
 
-    These callbacks must accept a single string argument representing the path
-    passed to the ``@import`` directive, and either return ``None`` to
+    These callbacks can accept one or two string arguments. The first argument
+    is the path that was passed to the ``@import`` directive; the second
+    (optional) argument is the previous resolved path, where the ``@import``
+    directive was found. The callbacks must either return ``None`` to
     indicate the path wasn't handled by that callback (to continue with others
     or fall back on internal ``libsass`` filesystem behaviour) or a list of
     one or more tuples, each in one of three forms:
@@ -494,7 +509,7 @@ def compile(**kwargs):
 
     .. code-block:: python
 
-        def my_importer(path):
+        def my_importer(path, prev):
             return [(path, '#' + path + ' { color: red; }')]
 
         sass.compile(
@@ -530,6 +545,10 @@ def compile(**kwargs):
        Added ``source_map_contents``, ``source_map_embed``,
        ``omit_source_map_url``, and ``source_map_root`` parameters.
 
+    .. versionadded:: 0.18.0
+        The importer callbacks can now take a second argument, the previously-
+        resolved path, so that importers can do relative path resolution.
+
     """
     modes = set()
     for mode_name in MODES:
@@ -545,13 +564,17 @@ def compile(**kwargs):
     precision = kwargs.pop('precision', 5)
     output_style = kwargs.pop('output_style', 'nested')
     if not isinstance(output_style, string_types):
-        raise TypeError('output_style must be a string, not ' +
-                        repr(output_style))
+        raise TypeError(
+            'output_style must be a string, not ' +
+            repr(output_style),
+        )
     try:
         output_style = OUTPUT_STYLES[output_style]
     except KeyError:
-        raise CompileError('{} is unsupported output_style; choose one of {}'
-                           ''.format(output_style, and_join(OUTPUT_STYLES)))
+        raise CompileError(
+            '{} is unsupported output_style; choose one of {}'
+            ''.format(output_style, and_join(OUTPUT_STYLES)),
+        )
     source_comments = kwargs.pop('source_comments', False)
     if source_comments in SOURCE_COMMENTS:
         if source_comments == 'none':
@@ -561,9 +584,11 @@ def compile(**kwargs):
             )
             source_comments = False
         elif source_comments in ('line_numbers', 'default'):
-            deprecation_message = ('you can simply pass True to '
-                                   "source_comments instead of " +
-                                   repr(source_comments))
+            deprecation_message = (
+                'you can simply pass True to '
+                "source_comments instead of " +
+                repr(source_comments)
+            )
             source_comments = True
         else:
             deprecation_message = (
@@ -579,8 +604,10 @@ def compile(**kwargs):
             FutureWarning,
         )
     if not isinstance(source_comments, bool):
-        raise TypeError('source_comments must be bool, not ' +
-                        repr(source_comments))
+        raise TypeError(
+            'source_comments must be bool, not ' +
+            repr(source_comments),
+        )
     fs_encoding = sys.getfilesystemencoding() or sys.getdefaultencoding()
 
     def _get_file_arg(key):
@@ -653,8 +680,10 @@ def compile(**kwargs):
             string = string.encode('utf-8')
         indented = kwargs.pop('indented', False)
         if not isinstance(indented, bool):
-            raise TypeError('indented must be bool, not ' +
-                            repr(source_comments))
+            raise TypeError(
+                'indented must be bool, not ' +
+                repr(source_comments),
+            )
         _check_no_remaining_kwargs(compile, kwargs)
         s, v = _sass.compile_string(
             string, output_style, source_comments, include_paths, precision,
@@ -771,9 +800,11 @@ SASS_SEPARATOR_SPACE = collections.namedtuple('SASS_SEPARATOR_SPACE', ())()
 SEPARATORS = frozenset((SASS_SEPARATOR_COMMA, SASS_SEPARATOR_SPACE))
 
 
-class SassList(collections.namedtuple(
-        'SassList', ('items', 'separator', 'bracketed'),
-)):
+class SassList(
+    collections.namedtuple(
+            'SassList', ('items', 'separator', 'bracketed'),
+    ),
+):
 
     def __new__(cls, items, separator, bracketed=False):
         items = tuple(items)
