@@ -1,8 +1,8 @@
 # -*- coding: utf-8 -*-
-from __future__ import with_statement
 
 import base64
 import contextlib
+import functools
 import glob
 import json
 import io
@@ -38,6 +38,13 @@ else:   # pragma: no cover (non-windows)
         return path
 
 
+@pytest.fixture(scope='session', autouse=True)
+def set_coverage_instrumentation():
+    if 'PWD' in os.environ:  # pragma: no branch
+        rcfile = os.path.join(os.environ['PWD'], '.coveragerc')
+        os.environ['COVERAGE_PROCESS_START'] = rcfile
+
+
 A_EXPECTED_CSS = '''\
 body {
   background-color: green; }
@@ -64,12 +71,8 @@ A_EXPECTED_MAP = {
     ),
 }
 
-with open('test/a.scss') as f:
-    A_EXPECTED_MAP_CONTENTS = dict(
-        A_EXPECTED_MAP, **{
-            'sourcesContent': [f.read()],
-        }
-    )
+with io.open('test/a.scss', newline='') as f:
+    A_EXPECTED_MAP_CONTENTS = dict(A_EXPECTED_MAP, sourcesContent=[f.read()])
 
 B_EXPECTED_CSS = '''\
 b i {
@@ -341,6 +344,40 @@ a {
         )
         assert ret == 'b i{font-size:20px}.foo-one-arg{color:blue}\n'
 
+    def test_importer_prev_path(self):
+        def importer(path, prev):
+            assert path in ('a', 'b')
+            if path == 'a':
+                assert prev == 'stdin'
+                return ((path, '@import "b";'),)
+            elif path == 'b':
+                assert prev == 'a'
+                return ((path, 'a { color: red; }'),)
+
+        ret = sass.compile(
+            string='@import "a";',
+            importers=((0, importer),),
+            output_style='compressed',
+        )
+        assert ret == 'a{color:red}\n'
+
+    def test_importer_prev_path_partial(self):
+        def importer(a_css, path, prev):
+            assert path in ('a', 'b')
+            if path == 'a':
+                assert prev == 'stdin'
+                return ((path, '@import "b";'),)
+            elif path == 'b':
+                assert prev == 'a'
+                return ((path, a_css),)
+
+        ret = sass.compile(
+            string='@import "a";',
+            importers=((0, functools.partial(importer, 'a { color: red; }')),),
+            output_style='compressed',
+        )
+        assert ret == 'a{color:red}\n'
+
     def test_importer_does_not_handle_returns_None(self):
         def importer_one(path):
             if path == 'one':
@@ -413,47 +450,53 @@ a {
         def importer(path):
             raise ValueError('Bad path: {}'.format(path))
 
-        with assert_raises_compile_error(RegexMatcher(
-                r'^Error: \n'
-                r'       Traceback \(most recent call last\):\n'
-                r'.+'
-                r'ValueError: Bad path: hi\n'
-                r'        on line 1 of stdin\n'
-                r'>> @import "hi";\n'
-                r'   --------\^\n',
-        )):
+        with assert_raises_compile_error(
+            RegexMatcher(
+                    r'^Error: \n'
+                    r'       Traceback \(most recent call last\):\n'
+                    r'.+'
+                    r'ValueError: Bad path: hi\n'
+                    r'        on line 1:9 of stdin\n'
+                    r'>> @import "hi";\n'
+                    r'   --------\^\n',
+            ),
+        ):
             sass.compile(string='@import "hi";', importers=((0, importer),))
 
     def test_importer_returns_wrong_tuple_size_zero(self):
         def importer(path):
             return ((),)
 
-        with assert_raises_compile_error(RegexMatcher(
-                r'^Error: \n'
-                r'       Traceback \(most recent call last\):\n'
-                r'.+'
-                r'ValueError: Expected importer result to be a tuple of '
-                r'length \(1, 2, 3\) but got 0: \(\)\n'
-                r'        on line 1 of stdin\n'
-                r'>> @import "hi";\n'
-                r'   --------\^\n',
-        )):
+        with assert_raises_compile_error(
+            RegexMatcher(
+                    r'^Error: \n'
+                    r'       Traceback \(most recent call last\):\n'
+                    r'.+'
+                    r'ValueError: Expected importer result to be a tuple of '
+                    r'length \(1, 2, 3\) but got 0: \(\)\n'
+                    r'        on line 1:9 of stdin\n'
+                    r'>> @import "hi";\n'
+                    r'   --------\^\n',
+            ),
+        ):
             sass.compile(string='@import "hi";', importers=((0, importer),))
 
     def test_importer_returns_wrong_tuple_size_too_big(self):
         def importer(path):
             return (('a', 'b', 'c', 'd'),)
 
-        with assert_raises_compile_error(RegexMatcher(
-                r'^Error: \n'
-                r'       Traceback \(most recent call last\):\n'
-                r'.+'
-                r'ValueError: Expected importer result to be a tuple of '
-                r"length \(1, 2, 3\) but got 4: \('a', 'b', 'c', 'd'\)\n"
-                r'        on line 1 of stdin\n'
-                r'>> @import "hi";\n'
-                r'   --------\^\n',
-        )):
+        with assert_raises_compile_error(
+            RegexMatcher(
+                    r'^Error: \n'
+                    r'       Traceback \(most recent call last\):\n'
+                    r'.+'
+                    r'ValueError: Expected importer result to be a tuple of '
+                    r"length \(1, 2, 3\) but got 4: \('a', 'b', 'c', 'd'\)\n"
+                    r'        on line 1:9 of stdin\n'
+                    r'>> @import "hi";\n'
+                    r'   --------\^\n',
+            ),
+        ):
             sass.compile(string='@import "hi";', importers=((0, importer),))
 
     def test_compile_string_deprecated_source_comments_line_numbers(self):
@@ -1348,24 +1391,27 @@ class RegexMatcher(object):
 class CustomFunctionsTest(unittest.TestCase):
 
     def test_raises(self):
-        with assert_raises_compile_error(RegexMatcher(
-                r'^Error: error in C function raises: \n'
-                r'       Traceback \(most recent call last\):\n'
-                r'.+'
-                r'AssertionError: foo\n'
-                r'        on line 1 of stdin, in function `raises`\n'
-                r'        from line 1 of stdin\n'
-                r'>> a { content: raises\(\); }\n'
-                r'   -------------\^\n$',
-        )):
+        with assert_raises_compile_error(
+            RegexMatcher(
+                    r'^Error: error in C function raises: \n'
+                    r'       Traceback \(most recent call last\):\n'
+                    r'.+'
+                    r'AssertionError: foo\n'
+                    r'        on line 1:14 of stdin, in function `raises`\n'
+                    r'        from line 1:14 of stdin\n'
+                    r'>> a { content: raises\(\); }\n'
+                    r'   -------------\^\n$',
+            ),
+        ):
             compile_with_func('a { content: raises(); }')
 
     def test_warning(self):
         with assert_raises_compile_error(
                 'Error: warning in C function returns_warning: '
                 'This is a warning\n'
-                '        on line 1 of stdin, in function `returns_warning`\n'
-                '        from line 1 of stdin\n'
+                '        on line 1:14 of stdin, '
+                'in function `returns_warning`\n'
+                '        from line 1:14 of stdin\n'
                 '>> a { content: returns_warning(); }\n'
                 '   -------------^\n',
         ):
@@ -1375,8 +1421,8 @@ class CustomFunctionsTest(unittest.TestCase):
         with assert_raises_compile_error(
                 'Error: error in C function returns_error: '
                 'This is an error\n'
-                '        on line 1 of stdin, in function `returns_error`\n'
-                '        from line 1 of stdin\n'
+                '        on line 1:14 of stdin, in function `returns_error`\n'
+                '        from line 1:14 of stdin\n'
                 '>> a { content: returns_error(); }\n'
                 '   -------------^\n',
         ):
@@ -1397,8 +1443,9 @@ class CustomFunctionsTest(unittest.TestCase):
                 '       - SassMap\n'
                 '       - SassWarning\n'
                 '       - SassError\n'
-                '        on line 1 of stdin, in function `returns_unknown`\n'
-                '        from line 1 of stdin\n'
+                '        on line 1:14 of stdin, '
+                'in function `returns_unknown`\n'
+                '        from line 1:14 of stdin\n'
                 '>> a { content: returns_unknown(); }\n'
                 '   -------------^\n',
         ):
@@ -1583,11 +1630,12 @@ def test_stack_trace_formatting():
         raise AssertionError('expected to raise CompileError')
     except sass.CompileError:
         tb = traceback.format_exc()
+    # TODO: https://github.com/sass/libsass/issues/3092
     assert tb.endswith(
         'CompileError: Error: Invalid CSS after "a{☃": expected "{", was ""\n'
-        '        on line 1 of stdin\n'
+        '        on line 1:5 of stdin\n'
         '>> a{☃\n'
-        '   --^\n\n',
+        '   ----^\n\n',
     )
 
 
