@@ -2,32 +2,34 @@
 ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 
 """
-from __future__ import with_statement
 
-import collections
 import io
 import os
 import os.path
 import re
+import warnings
 
 from six import string_types
 
 from sass import compile
+from sassutils._compat import collections_abc
 
 __all__ = 'SUFFIXES', 'SUFFIX_PATTERN', 'Manifest', 'build_directory'
 
 
-#: (:class:`collections.Set`) The set of supported filename suffixes.
-SUFFIXES = frozenset(['sass', 'scss'])
+#: (:class:`frozenset`) The set of supported filename suffixes.
+SUFFIXES = frozenset(('sass', 'scss'))
 
 #: (:class:`re.RegexObject`) The regular expression pattern which matches to
 #: filenames of supported :const:`SUFFIXES`.
 SUFFIX_PATTERN = re.compile('[.](' + '|'.join(map(re.escape, SUFFIXES)) + ')$')
 
 
-def build_directory(sass_path, css_path, output_style='nested',
-                    _root_sass=None, _root_css=None):
-    """Compiles all SASS/SCSS files in ``path`` to CSS.
+def build_directory(
+    sass_path, css_path, output_style='nested',
+    _root_sass=None, _root_css=None, strip_extension=False,
+):
+    """Compiles all Sass/SCSS files in ``path`` to CSS.
 
     :param sass_path: the path of the directory which contains source files
                       to compile
@@ -39,7 +41,7 @@ def build_directory(sass_path, css_path, output_style='nested',
                          ``'compact'``, ``'compressed'``
     :type output_style: :class:`str`
     :returns: a dictionary of source filenames to compiled CSS filenames
-    :rtype: :class:`collections.Mapping`
+    :rtype: :class:`collections.abc.Mapping`
 
     .. versionadded:: 0.6.0
        The ``output_style`` parameter.
@@ -57,10 +59,14 @@ def build_directory(sass_path, css_path, output_style='nested',
             if name[0] == '_':
                 # Do not compile if it's partial
                 continue
+            if strip_extension:
+                name, _ = os.path.splitext(name)
             css_fullname = os.path.join(css_path, name) + '.css'
-            css = compile(filename=sass_fullname,
-                          output_style=output_style,
-                          include_paths=[_root_sass])
+            css = compile(
+                filename=sass_fullname,
+                output_style=output_style,
+                include_paths=[_root_sass],
+            )
             with io.open(
                 css_fullname, 'w', encoding='utf-8', newline='',
             ) as css_file:
@@ -69,92 +75,154 @@ def build_directory(sass_path, css_path, output_style='nested',
                 os.path.relpath(css_fullname, _root_css)
         elif os.path.isdir(sass_fullname):
             css_fullname = os.path.join(css_path, name)
-            subresult = build_directory(sass_fullname, css_fullname,
-                                        output_style=output_style,
-                                        _root_sass=_root_sass,
-                                        _root_css=_root_css)
+            subresult = build_directory(
+                sass_fullname, css_fullname,
+                output_style=output_style,
+                _root_sass=_root_sass,
+                _root_css=_root_css,
+                strip_extension=strip_extension,
+            )
             result.update(subresult)
     return result
 
 
 class Manifest(object):
-    """Building manifest of SASS/SCSS.
+    """Building manifest of Sass/SCSS.
 
-    :param sass_path: the path of the directory that contains SASS/SCSS
+    :param sass_path: the path of the directory that contains Sass/SCSS
                       source files
     :type sass_path: :class:`str`, :class:`basestring`
     :param css_path: the path of the directory to store compiled CSS
                      files
     :type css_path: :class:`str`, :class:`basestring`
-
+    :param strip_extension: whether to remove the original file extension
+    :type strip_extension: :class:`bool`
     """
 
     @classmethod
     def normalize_manifests(cls, manifests):
         if manifests is None:
             manifests = {}
-        elif isinstance(manifests, collections.Mapping):
+        elif isinstance(manifests, collections_abc.Mapping):
             manifests = dict(manifests)
         else:
-            raise TypeError('manifests must be a mapping object, not ' +
-                            repr(manifests))
+            raise TypeError(
+                'manifests must be a mapping object, not ' +
+                repr(manifests),
+            )
         for package_name, manifest in manifests.items():
             if not isinstance(package_name, string_types):
-                raise TypeError('manifest keys must be a string of package '
-                                'name, not ' + repr(package_name))
+                raise TypeError(
+                    'manifest keys must be a string of package '
+                    'name, not ' + repr(package_name),
+                )
             if isinstance(manifest, Manifest):
                 continue
             elif isinstance(manifest, tuple):
                 manifest = Manifest(*manifest)
+            elif isinstance(manifest, collections_abc.Mapping):
+                manifest = Manifest(**manifest)
             elif isinstance(manifest, string_types):
                 manifest = Manifest(manifest)
             else:
                 raise TypeError(
                     'manifest values must be a sassutils.builder.Manifest, '
                     'a pair of (sass_path, css_path), or a string of '
-                    'sass_path, not ' + repr(manifest)
+                    'sass_path, not ' + repr(manifest),
                 )
             manifests[package_name] = manifest
         return manifests
 
-    def __init__(self, sass_path, css_path=None, wsgi_path=None):
+    def __init__(
+            self,
+            sass_path,
+            css_path=None,
+            wsgi_path=None,
+            strip_extension=None,
+    ):
         if not isinstance(sass_path, string_types):
-            raise TypeError('sass_path must be a string, not ' +
-                            repr(sass_path))
+            raise TypeError(
+                'sass_path must be a string, not ' +
+                repr(sass_path),
+            )
         if css_path is None:
             css_path = sass_path
         elif not isinstance(css_path, string_types):
-            raise TypeError('css_path must be a string, not ' +
-                            repr(css_path))
+            raise TypeError(
+                'css_path must be a string, not ' +
+                repr(css_path),
+            )
         if wsgi_path is None:
             wsgi_path = css_path
         elif not isinstance(wsgi_path, string_types):
-            raise TypeError('wsgi_path must be a string, not ' +
-                            repr(wsgi_path))
+            raise TypeError(
+                'wsgi_path must be a string, not ' +
+                repr(wsgi_path),
+            )
+        if strip_extension is None:
+            warnings.warn(
+                '`strip_extension` was not specified, defaulting to `False`.\n'
+                'In the future, `strip_extension` will default to `True`.',
+                FutureWarning,
+            )
+            strip_extension = False
+        elif not isinstance(strip_extension, bool):
+            raise TypeError(
+                'strip_extension must be bool not {!r}'.format(
+                    strip_extension,
+                ),
+            )
         self.sass_path = sass_path
         self.css_path = css_path
         self.wsgi_path = wsgi_path
+        self.strip_extension = strip_extension
 
     def resolve_filename(self, package_dir, filename):
-        """Gets a proper full relative path of SASS source and
+        """Gets a proper full relative path of Sass source and
         CSS source that will be generated, according to ``package_dir``
         and ``filename``.
 
         :param package_dir: the path of package directory
         :type package_dir: :class:`str`, :class:`basestring`
-        :param filename: the filename of SASS/SCSS source to compile
+        :param filename: the filename of Sass/SCSS source to compile
         :type filename: :class:`str`, :class:`basestring`
         :returns: a pair of (sass, css) path
         :rtype: :class:`tuple`
 
         """
         sass_path = os.path.join(package_dir, self.sass_path, filename)
+        if self.strip_extension:
+            filename, _ = os.path.splitext(filename)
         css_filename = filename + '.css'
         css_path = os.path.join(package_dir, self.css_path, css_filename)
         return sass_path, css_path
 
+    def unresolve_filename(self, package_dir, filename):
+        """Retrieves the probable source path from the output filename.  Pass
+        in a .css path to get out a .scss path.
+
+        :param package_dir: the path of the package directory
+        :type package_dir: :class:`str`
+        :param filename: the css filename
+        :type filename: :class:`str`
+        :returns: the scss filename
+        :rtype: :class:`str`
+        """
+        filename, _ = os.path.splitext(filename)
+        if self.strip_extension:
+            for ext in ('.scss', '.sass'):
+                test_path = os.path.join(
+                    package_dir, self.sass_path, filename + ext,
+                )
+                if os.path.exists(test_path):
+                    return filename + ext
+            else:  # file not found, let it error with `.scss` extension
+                return filename + '.scss'
+        else:
+            return filename
+
     def build(self, package_dir, output_style='nested'):
-        """Builds the SASS/SCSS files in the specified :attr:`sass_path`.
+        """Builds the Sass/SCSS files in the specified :attr:`sass_path`.
         It finds :attr:`sass_path` and locates :attr:`css_path`
         as relative to the given ``package_dir``.
 
@@ -165,7 +233,7 @@ class Manifest(object):
                              ``'expanded'``, ``'compact'``, ``'compressed'``
         :type output_style: :class:`str`
         :returns: the set of compiled CSS filenames
-        :rtype: :class:`collections.Set`
+        :rtype: :class:`frozenset`
 
         .. versionadded:: 0.6.0
            The ``output_style`` parameter.
@@ -175,17 +243,20 @@ class Manifest(object):
         css_path = os.path.join(package_dir, self.css_path)
         css_files = build_directory(
             sass_path, css_path,
-            output_style=output_style
+            output_style=output_style,
+            strip_extension=self.strip_extension,
         ).values()
-        return frozenset(os.path.join(self.css_path, filename)
-                         for filename in css_files)
+        return frozenset(
+            os.path.join(self.css_path, filename)
+            for filename in css_files
+        )
 
     def build_one(self, package_dir, filename, source_map=False):
-        """Builds one SASS/SCSS file.
+        """Builds one Sass/SCSS file.
 
         :param package_dir: the path of package directory
         :type package_dir: :class:`str`, :class:`basestring`
-        :param filename: the filename of SASS/SCSS source to compile
+        :param filename: the filename of Sass/SCSS source to compile
         :type filename: :class:`str`, :class:`basestring`
         :param source_map: whether to use source maps.  if :const:`True`
                            it also write a source map to a ``filename``
@@ -200,7 +271,8 @@ class Manifest(object):
 
         """
         sass_filename, css_filename = self.resolve_filename(
-            package_dir, filename)
+            package_dir, filename,
+        )
         root_path = os.path.join(package_dir, self.sass_path)
         css_path = os.path.join(package_dir, self.css_path, css_filename)
         if source_map:
@@ -208,7 +280,8 @@ class Manifest(object):
             css, source_map = compile(
                 filename=sass_filename,
                 include_paths=[root_path],
-                source_map_filename=source_map_path  # FIXME
+                source_map_filename=source_map_path,  # FIXME
+                output_filename_hint=css_path,
             )
         else:
             css = compile(filename=sass_filename, include_paths=[root_path])
